//
//  StatusesTableViewController.swift
//  Tusker
//
//  Created by Shadowfacts on 8/15/18.
//  Copyright © 2018 Shadowfacts. All rights reserved.
//

import UIKit
import Pachyderm

class TimelineTableViewController: UITableViewController {

    static func create(for timeline: Timeline) -> UIViewController {
        guard let navigationController = UIStoryboard(name: "Timeline", bundle: nil).instantiateInitialViewController() as? UINavigationController,
            let timelineController = navigationController.topViewController as? TimelineTableViewController else { fatalError() }
        timelineController.timeline = timeline
        
        let title: String
        switch timeline {
        case .home:
            title = "Home"
        case let .public(local):
            title = local ? "Local" : "Federated"
        case let .tag(hashtag):
            title = "#\(hashtag)"
        case .list:
            title = "List"
        case .direct:
            title = "Direct"
        }
        navigationController.tabBarItem.title = title
        timelineController.navigationItem.title = title
        
        return navigationController
    }
    
    var timeline: Timeline!
    
    var statuses: [Status] = [] {
        didSet {
            DispatchQueue.main.async {
                self.tableView.reloadData()
            }
        }
    }
    
    var newer: RequestRange?
    var older: RequestRange?
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        tableView.rowHeight = UITableView.automaticDimension
        tableView.estimatedRowHeight = 140
        
        tableView.register(UINib(nibName: "StatusTableViewCell", bundle: nil), forCellReuseIdentifier: "statusCell")
        
        guard MastodonController.shared.client?.accessToken != nil else { return }
        MastodonController.shared.client.getStatuses(timeline: timeline) { response in
            guard case let .success(statuses, pagination) = response else { fatalError() }
            self.statuses = statuses
            self.newer = pagination?.newer
            self.older = pagination?.older
        }
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        
        for cell in tableView.visibleCells {
            if let cell = cell as? PreferencesAdaptive {
                cell.updateUIForPreferences()
            }
        }
    }

    /*
     // MARK: - Navigation
     
     // In a storyboard-based application, you will often want to do a little preparation before navigation
     override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
     // Get the new view controller using segue.destination.
     // Pass the selected object to the new view controller.
     }
     */
    
    // MARK: - Table view data source

    override func numberOfSections(in tableView: UITableView) -> Int {
        return 1
    }

    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return statuses.count
    }

    
    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        guard let cell = tableView.dequeueReusableCell(withIdentifier: "statusCell", for: indexPath) as? StatusTableViewCell else { fatalError() }
        
        let status = statuses[indexPath.row]
        
        cell.updateUI(for: status)
        
        cell.delegate = self

        return cell
    }
    
    override func tableView(_ tableView: UITableView, willDisplay cell: UITableViewCell, forRowAt indexPath: IndexPath) {
        if indexPath.row == statuses.count - 1 {
            guard let older = older else { return }
            
            MastodonController.shared.client.getStatuses(timeline: timeline, range: older) { response in
                guard case let .success(newStatuses, pagination) = response else { fatalError() }
                self.older = pagination?.older
                self.statuses.append(contentsOf: newStatuses)
            }
        }
    }

    @IBAction func refreshStatuses(_ sender: Any) {
        guard let newer = newer else { return }
        
        MastodonController.shared.client.getStatuses(timeline: timeline, range: newer) { response in
            guard case let .success(newStatuses, pagination) = response else { fatalError() }
            self.newer = pagination?.newer
            self.statuses.insert(contentsOf: newStatuses, at: 0)
            DispatchQueue.main.async {
                self.refreshControl?.endRefreshing()
                
                // maintain the current position in the list (don't scroll to the top)
                self.tableView.scrollToRow(at: IndexPath(row: newStatuses.count, section: 0), at: .top, animated: false)
            }
        }
    }
    
}

extension TimelineTableViewController: StatusTableViewCellDelegate {
    func updatedStatus(for cell: StatusTableViewCell, status: Status) {
        let indexPath = tableView.indexPath(for: cell)!
        statuses[indexPath.row] = status
    }
}
extension TimelineTableViewController: LargeImageViewControllerDelegate {}
